#!/bin/sh

# download and package chrome or opera
# 1st version

[ $(id -u) -ne 0 ] && exec sudo -A ${0} ${@}

export TEXTDOMAIN=dl_chrome_opera.sh
export OUTPUT_CHARSET=UTF-8

# func strings
export ERR_TITLE=$(gettext 'Download Chrome/Opera - error')
#export TTLE=$(gettext 'file exists')
export EX=$(gettext ' already exists in /mnt/home. Load it or move it! Exiting.')
export TTLC=$(gettext ' needs GTK 3.0')
export MSGC=$(gettext ' requires gtk+3-3.0 to run. If you wish to continue you must download gtk 3 from the package manager later for the browser to run. Press Yes to continue or No to quit.')
export OTTL=$(gettext 'Opera is an older version')
. /etc/rc.d/PUPSTATE

export PUP_HOME

# gui for errors
function init_error() {
	MSG="$1"
export ERR='<window title="'$ERR_TITLE'" icon-name="gtk-no">
	<vbox>
	'"`/usr/lib/gtkdialog/xml_info fixed /usr/share/pixmaps/dl_chrome_opera.svg 60 "$MSG"`"' 
	  <frame>
	    <hbox homogeneous="true">
			<button ok></button>
		</hbox>
	  </frame>
	</vbox>
</window>'
. /usr/lib/gtkdialog/xml_info gtk
gtkdialog -p ERR -c && exit 1
}

# gui for chrome and opera as of 2017/06 need gtk3 .. ugh
function other_error() {
	TTLC="$2"
	MSGC="$1"
	BR=$3
export GTKERR='<window title="'${BR}${TTLC}'" icon-name="gtk-info">
	<vbox>
	'"`/usr/lib/gtkdialog/xml_info fixed /usr/share/pixmaps/dl_chrome_opera.svg 60 "${BR}${MSGC}"`"' 
	  <frame>
	    <hbox homogeneous="true">
			<button yes></button><button no></button>
		</hbox>
	  </frame>
	</vbox>
</window>'
. /usr/lib/gtkdialog/xml_info gtk
eval $(gtkdialog -p GTKERR -c)
[ "$EXIT" = 'No' ] && return 1 || return 0
}

# check if an sfs of similar name is loaded
function is_sfs_loaded() {
	local B_SFS_LOADED
	[ -f /tmp/sfs_load/loop_device_list ] && B_SFS_LOADED=`grep -o "${1}.*" /tmp/sfs_load/loop_device_list`
	if [ -z "$B_SFS_LOADED" ]; then
		return 1 #not loaded
	fi
	return 0
}

# check if a pet of similar name is installed
function is_pet_installed() {
	local B_PET_INSTALLED
	[ -f ~/.packages/user-installed-packages ] && B_PET_INSTALLED=`grep -o "${1}\-.*\.pet" ~/.packages/user-installed-packages`
	B_PET_INSTALLED=${B_PET_INSTALLED##*\|}
	if [ -z "$B_PET_INSTALLED" ]; then
		return 1 #not installed
	fi
	return 0
}

# check if sfs is in /mnt/home
function check_sfs_exists() {
	BRS_FILE="$1"
	is_sfs_loaded $BRS_FILE && return 0
	if [ -f "/initrd/${PUP_HOME}/${BRS_FILE}" ];then
		kill -9 $apid
		init_error "$BRS_FILE $EX"
	fi
}

# unload
unload_sfs() {
	sfs_load
}

#uninstall
uninstall_pet() {
	petget
}
export -f init_error
export -f other_error
export -f is_sfs_loaded
export -f is_pet_installed
export -f check_sfs_exists
export -f unload_sfs
export -f uninstall_pet

GTK3=''
find /usr -maxdepth 2 -type d -name 'gtk-*'|grep 'lib'|grep -q '3' && GTK3='yes'

which strings 2>&1 >/dev/null && export STRINGS=strings || export STRINGS='busybox strings'

# check connection
NOCON=$(gettext 'It seems you are not connected to the internet. Exiting.')
#check_internet 2>&1 > /dev/null || init_error "$NOCON"

# tmp
TMP=${TMP:-/tmp}

# check install type
state=true
MSG3=$(gettext "Choose if you want a sfs package or pet package")
case $PUPMODE in
	2|3|5|77)state=false
			MSG3=$(gettext "Your type of install only supports PET packages")
			deff=true;;
			*)deff=false;;
esac

# check arch
arch=`uname -m`
case $arch in
	x86_64)CHROME=true
		OPERA=true
		MSG1=$(gettext "You can download the latest Chrome or Opera browser and create and install a Puppy package.")
		MSG2=$(gettext "Just hit the button of the browser you wish to install and the download will start")
		DEBARCH=amd64
		CARCH=$arch;;
	*)exit;;
esac

# find icons
PETICON=/usr/share/pixmaps/puppy/package_pet.svg
SFSICON=/usr/share/pixmaps/puppy/package_sfs.svg
ICONTHEME=`grep -i 'icon-theme' ~/.gtkrc-2.0|sed -e 's/^gtk-icon-theme-name = //' -e 's/\"//g'`
[ -e "/usr/share/icons/$ICONTHEME/48/mimetypes/application-pet.svg" ] &&\
	PETICON="/usr/share/icons/$ICONTHEME/48/mimetypes/application-pet.svg"
[ -e "/usr/share/icons/$ICONTHEME/48/mimetypes/application-x-squashfs-image.svg" ] &&\
	SFSICON="/usr/share/icons/$ICONTHEME/48/mimetypes/application-x-squashfs-image.svg"
	
# check if installed ####
# but wait..
gtkdialog-splash -close never -bg CornflowerBlue -text "$(gettext 'Please wait... checking latest versions')" &
export apid=$!
# chrome
if [ "$CARCH" = 'x86_64' ];then
  read xx yy CUR_CVERSION <<< $(google-chrome-stable --version 2>/dev/null)
  NEW_CVERSION=$(wget -qO- https://dl.google.com/linux/direct/google-chrome-stable_current_x86_64.rpm | head -c96 | $STRINGS | rev | awk -F"[:-]" '/emorhc/ { print $1 "-" $2 }' | rev)
  if echo $NEW_CVERSION | grep -Fq '-'; then
    GOOGLEREVISION=${NEW_CVERSION#*-}
  NEW_CVERSION=${NEW_CVERSION%-*}
  else
    GOOGLEREVISION=""
  fi
fi
# opera
NEW_OVERSION=$(wget -qO- http://deb.opera.com/opera/dists/stable/non-free/binary-$DEBARCH/Packages.gz | gzip -d | grep -A1 -x "Package: opera-stable" | sed -n "/Version/s/.* //p")
CUR_OVERSION=$(opera --version 2>/dev/null)

# common
BGUI=''
for CUR in google-chrome opera
do
	case "$CUR" in
		google-chrome)VER="$CUR_CVERSION";NVER="$NEW_CVERSION"
			[ "$CARCH" = 'i386' ] && continue;; # no 32 bit
		opera)VER="$CUR_OVERSION";NVER="$NEW_OVERSION";;
	esac
	# check sfs exists in /mnt/home and not loaded
	XVER=''
	echo ${CUR}-${NVER}${XVER}-${CARCH}.sfs
	# this stops the program on the first match
	check_sfs_exists ${CUR}-${NVER}${XVER}-${CARCH}.sfs
	# check latest installed
	echo "$VER" "$NVER"
	if [ "$VER" = "$NVER" ];then
		case "$CUR" in
			google-chrome)CHROME=false;;
			opera)OPERA=false;;
		esac
		continue
	fi
	case "$CUR" in
		google-chrome) [ -n "$VER" ] && LC=false || LC=true;;
		opera) [ -n "$VER" ] && OC=false || OC=true;;
	esac
	[ -z "$VER" ] && continue 
	#check if sfs or pet
	is_pet_installed $CUR
	pret=$?
	if [ $pret -eq 1 ];then
		is_sfs_loaded $CUR
		sret=$?
		if [ $sret -eq 0 ];then
			ICON=$SFSICON
			ACT=unload
			PROG=sfs_load
		fi
	else
		ICON=$PETICON
		ACT=uninstall
		PROG=petget
	fi
	# add to main gui
	TBASE1=$(gettext ', an older version is installed. Uninstall it with ')
	TBASE2=$(gettext ' - button on left. ')
	BGUI="$BGUI"'<hbox space-expand="true" space-fill="true">
			<button space-expand="false" space-fill="true">
			  <input file>'$ICON'</input>
			  <width>32</width><height>32</height>
			  <action function="exit">'$ACT'</action>
			</button>
			<text use-markup="true" space-expand="true" space-fill="false" justify="2">
			  <label>"<i>'${CUR}-${VER}${TBASE1}${PROG}${TBASE2}'</i>"</label>
			</text>
		</hbox>'
		echo $CHROME $OPERA
done
[ -n "$BGUI" ] && BGUI='<vbox space-expand="true" space-fill="false">'$BGUI'</vbox>
	<hseparator></hseparator>'
####

SPOT_MSG=$(gettext "All browsers will run as unprivileged user Spot")
if [ "$CHROME" = 'true' ]; then
  CH='<button image-position="top" theme-icon-size="32" tooltip-text="version '$NEW_CVERSION'">
			<label>Chrome</label>
			<variable>btnch</variable>
			<input file icon="chrome"></input>
			<sensitive>'$LC'</sensitive>
		</button>'
else
  CH=''
fi
if [ "$OPERA" = 'true' ]; then
  OP='<button image-position="top" theme-icon-size="32" tooltip-text="version '$NEW_OVERSION'">
			<label>Opera</label>
			<variable>btnop</variable>
			<input file icon="opera"></input>
			<sensitive>'$OC'</sensitive>
		</button>'
else
  OP=''
fi
ALT=''
CHOICE='<frame>
	  <hbox homogeneous="true">
	    <text><label>'"$MSG3"'</label></text>
	  </hbox>
	  <hbox homogeneous="true">
	      <pixmap>
	        <width>32</width>
			<height>32</height>
	        <input file>'"$SFSICON"'</input>
	        <sensitive>'$state'</sensitive>
	      </pixmap>		  
	      <radiobutton>
			<label>SFS</label>
			<sensitive>'$state'</sensitive>
			<variable>RADIOBUTTON2</variable>
		  </radiobutton>
		  <pixmap>
	        <width>32</width>
			<height>32</height>
	        <input file>'"$PETICON"'</input>
	      </pixmap>
	      <radiobutton>
			<label>PET</label>
			<variable>RADIOBUTTON1</variable>
			<default>'$deff'</default>
		  </radiobutton>
	      
	  </hbox>
	  </frame>'
if [ "$CHROME" = 'false' -a "$OPERA" = 'false' ];then
  ALTMSG=$(gettext 'Both offered browsers are up to date, so the choice is simple!')
  ALT='<text><label>'$ALTMSG'</label></text>';MSG2=$(gettext 'Congratulations');CHOICE=''
fi

if [ "$CHROME" = 'true' -a "$OPERA" = 'false' ];then
  MSG2=$(gettext 'Only Chrome is available. Opera is installed and up to date')
elif [ "$CHROME" = 'false' -a "$OPERA" = 'true' ];then
  MSG2=$(gettext 'Only Opera is available. Chrome is installed and up to date')
fi

kill -9 $apid

# main dialog
export DLG='<window title="'$(gettext 'Download a browser')'" icon-name="internet">
	<vbox width-request="500">
	'"`/usr/lib/gtkdialog/xml_info fixed /usr/share/pixmaps/dl_chrome_opera.svg 60 "$MSG1"`"' 
	  '$CHOICE'
	  <frame>
	  '$BGUI'
	  <hbox homogeneous="true">
	    <text><label>'"$MSG2"'</label></text>
	  </hbox>
	  <hbox homogeneous="true">  
	      '$CH'
	      '$OP'
	      '$ALT'
	  </hbox>
	  </frame>
	  <hbox homogeneous="true">
	    <text use-markup="true">
	      <label>"<span fgcolor='"'darkgreen'"'><i>'"$SPOT_MSG"'</i></span>"</label>
	    </text>
	  </hbox>
	  <hbox>
	    <button cancel></button>
	  </hbox>
	</vbox>
</window>'

. /usr/lib/gtkdialog/xml_info gtk
PKG_TYPE='pet'
case $1 in
 -d)echo "$DLG";;
 *)eval $(gtkdialog -p DLG);;
esac
# package type
if [ "$RADIOBUTTON2" = 'true' ];then
	PKG_TYPE='sfs'
fi
case $EXIT in
	unload)unload_sfs
	exec $0;;
	uninstall)uninstall_pet
	exec $0;;
esac
PKG=$EXIT
case $PKG in
	Chrome) if [ "$GTK3" != 'yes' ];then 
			  other_error "$MSGC" "$TTLC" $PKG
			  [ $? -ne 0 ] && exit 0
		    fi
		    PKG_TYPE=$PKG_TYPE urxvt -bg green -fg blue --geometry 80x10+1+1 -e latest-chrome.sh;;
	Opera)  if [ "$GTK3" != 'yes' ];then 
			    other_error "$MSGC" "$TTLC" $PKG
			    [ $? -ne 0 ] && exit 0
		    fi
		    PKG_TYPE=$PKG_TYPE urxvt -bg green -fg blue --geometry 80x10+1+1 -e latest-opera.sh;;
	*)exit;;
esac
FMSG=$(gettext "failed to be generated.")
SMSG=$(gettext "is located in /tmp. You can choose to save it before you shut down your computer or install it in the next step.")
# find the package
PACKAGE=`ls ${TMP} | grep -io "${PKG}.*" | grep  "${PKG_TYPE}$" | tail -1` # tail in case of older pkg
PACKAGE=`ls ${TMP} | grep "${PACKAGE}"` # chrome
[ -z "$PACKAGE" -o ! -f "${TMP}/$PACKAGE" ] &&  gtkdialog-splash -bg orange -close box -timeout 6 \
-text "$PKG $PKG_TYPE $FMSG" && exit 1
gtkdialog-splash -bg green -close box -timeout 10 \
-text "$PACKAGE $SMSG"
# install
case $PACKAGE in 
	*.pet)petget +${TMP}/${PACKAGE};;
	*.sfs)sfs_load +${TMP}/${PACKAGE};;
esac

# END
