#!/bin/bash
# latest-chrome Version 1.3.3
# hacked for puppy by 01micko

# This script will find the latest Google Chrome binary package,
# download it and repackage it into Slackware format.

# I don't use Chrome for regular browsing but it is handy for
# comparative tests against Vivaldi. :P

# Copyright 2016 Ruari Oedegaard, Oslo, Norway
# All rights reserved.
#
# Redistribution and use of this script, with or without modification, is
# permitted provided that the following conditions are met:
#
# 1. Redistributions of this script must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# HOLDER BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


# source browser.func
. /usr/share/dl_browser/browser.func

# package type
PKG_TYPE=${PKG_TYPE:-pet}
echo "Package type selected is $PKG_TYPE"
sleep 3

# Use the architecture of the current machine or whatever the user has
# set externally
ARCH=${ARCH:-$(uname -m)}
if [ ! "$ARCH" = "x86_64" ]; then
  echo "The architecture $ARCH is not supported." >&2
  exit 1
fi

# Work out the latest stable Google Chrome if VERSION is unset
VERSION=${VERSION:-$(wget -qO- https://dl.google.com/linux/direct/google-chrome-stable_current_x86_64.rpm | head -c96 | $STRINGS | rev | awk -F"[:-]" '/emorhc/ { print $1 "-" $2 }' | rev)}

# Error out if $VERISON is unset, e.g. because previous command failed
if [ -z $VERSION ]; then
  echo "Could not work out the latest version; exiting" >&2
  exit 1
fi

if echo $VERSION | grep -Fq '-'; then
  GOOGLEREVISION=${VERSION#*-}
  VERSION=${VERSION%-*}
else
  GOOGLEREVISION=""
fi
echo "new: $VERSION"

CWD=$(pwd)
TMP=${TMP:-/tmp}
REPACKDIR=$TMP/repackage-google-chrome

# Since packaging is about to begin errors become more important now,
# so exit if things fail.
set -eu

# If the repackage is already present from the past, clear it down
# and re-create it.
if [ -d "$REPACKDIR" ]; then
  rm -fr "$REPACKDIR"
fi

mkdir -p "$REPACKDIR"/pkg/src
mkdir -p "$REPACKDIR"/src

# Check if the current directory contains the correct Google Chrome
# binary package, otherwise download it. Then check that it matches the
# version number defined.
if [ -e google-chrome-stable-${VERSION}-${GOOGLEREVISION}.${ARCH}.rpm ]; then
  (
    cd "$REPACKDIR/src/"
    ln -s "$CWD/google-chrome-stable-${VERSION}-${GOOGLEREVISION}.${ARCH}.rpm" google-chrome-stable-${VERSION}-${GOOGLEREVISION}.${ARCH}.rpm
  )
else
  echo "Downloading Google Chrome ${VERSION} for ${ARCH}"
  if echo $GOOGLEREVISION | grep -q .; then
    wget https://dl.google.com/linux/chrome/rpm/stable/${ARCH}/google-chrome-stable-${VERSION}-${GOOGLEREVISION}.${ARCH}.rpm -P "$REPACKDIR/src/"
  else
    wget https://dl.google.com/linux/direct/google-chrome-stable_current_${ARCH}.rpm -O "$REPACKDIR/src/google-chrome-stable_current_${ARCH}.rpm"
    DOWNLOADVERSION=$(head -c96 "$REPACKDIR/src/google-chrome-stable_current_${ARCH}.rpm" | $STRINGS | rev | awk -F"[:-]" '/emorhc/ { print $2 }' | rev)
    if [ ! "$VERSION" = "$DOWNLOADVERSION" ]; then
      echo "The version downloaded ($DOWNLOADVERSION) is different from the version defined ($VERSION)" >&2
      exit 1
    fi
  fi
fi

# Now we have all the sources in place, switch to the package directory
# and start setting things up.
cd "$REPACKDIR/pkg"

# Extract the contents of the Google Chrome binary package
if [ -x /usr/bin/bsdtar ]; then
  bsdtar xf ../src/google-chrome-stable*${ARCH}.rpm
elif [ -x /usr/bin/rpm2cpio ]; then
  rpm2cpio ../src/google-chrome-stable*${ARCH}.rpm | cpio --quiet -id
else
  # Since the user has not installed libarchive or rpm, use a hack to extract the rpm contents
  RPMHDRLGTH=$(LANG=C grep -abom1 '.7zXZ\|BZh9' ../src/google-chrome-stable*${ARCH}.rpm)
  case "$RPMHDRLGTH" in
    *7zXZ) COMPRESSOR=xz ;;
    *BZh9) COMPRESSOR=bzip2 ;;
    *)     echo "Unknown compression type in rpm!" >&2; exit 1 ;;
  esac
  tail -c+$[${RPMHDRLGTH%:*}+1] ../src/google-chrome-stable*${ARCH}.rpm | $COMPRESSOR -d | cpio --quiet -id
fi

# Remove ./etc as it contains a cron job for updating rpm systems
rm -fr etc

# Replace google-chrome executable symlink, with a relative one
if [ -h usr/bin/google-chrome ]; then
  rm usr/bin/google-chrome
  (
    cd usr/bin
    ln -fs ../../opt/google/chrome/google-chrome google-chrome
  )
fi

# Symlink desktop file
if [ ! -e usr/share/applications/google-chrome.desktop ]; then
  mkdir -p usr/share/applications
  (
    cd usr/share/applications
    ln -s ../../../opt/google/chrome/google-chrome.desktop google-chrome.desktop
  )
fi

# Symlink desktop environment icons
for png in opt/google/chrome/product_logo_*.png; do
  pngsize="${png##*/product_logo_}"
  mkdir -p usr/share/icons/hicolor/${pngsize%.png}x${pngsize%.png}/apps
  (
    cd usr/share/icons/hicolor/${pngsize%.png}x${pngsize%.png}/apps/
    ln -s ../../../../../../opt/google/chrome/product_logo_${pngsize} google-chrome.png
  )
done

# fix.desktop
sed -i -e 's/Icon=.*/Icon=chrome.svg/' \
		-e "s/Exec=/Exec=run-as-spot /" \
		-e 's/ %U//' \
		-e 's/Categories=.*/Categories=WebBrowser;/' \
		usr/share/applications/*.desktop

# Make sure the file permissions are ok
chmod -R u+w,go+r-w,a-s .

# chrome-sandbox needs to be setuid root for Google Chrome to run
chmod 4711 opt/google/chrome/chrome-sandbox

# package as pet or sfs
( 
cd ../ 
if [ "$PKG_TYPE" = "pet" ]; then
  p=google-chrome-${VERSION}-${ARCH}
  mv pkg ${p}
  chmod 755 $p
  DESC="Google-Chrome is a web browser based on the blink engine"
  DEPS=""
  CAT=Internet
  echo "Please wait while constructing ${p}.pet ..."
  dir2pet -x -s -w="$DESC" -d="$DEPS" -c="$CAT" -p=${p} 2>&1 >/dev/null
  cp ${p}.pet /tmp  
else # sfs
  mksquashfs pkg google-chrome-${VERSION}-${ARCH}.sfs
  cp google-chrome-${VERSION}-${ARCH}.sfs /tmp
fi
)

# cleanup
cd ~
rm -r $REPACKDIR

# And ... we're done! ;)
echo 'google-chrome was successfully packaged'
exit 0
