#!/bin/bash
# latest-opera Version 2.0RC3
# hacked for puppy by 01micko

# This script will find the latest Opera stable binary package,
# download it and repackage it into Slackware format.

# Copyright 2015 Ruari Oedegaard, Oslo, Norway
# All rights reserved.
#
# Redistribution and use of this script, with or without modification, is
# permitted provided that the following conditions are met:
#
# 1. Redistributions of this script must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# HOLDER BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

available () {
  command -v "$1" >/dev/null 2>&1
}

# Binutils or libarchive (bsdtar) is needed for this script to run
if ! available ar; then
  if ! available bsdtar; then
    echo "You must install GNU Binutils or libarchive to use this script" >&2
    exit 1
  fi
  # source the special func lib that contains an 'ar' wrapper with bsdtar
  . /usr/share/dl_browser/s.browser.func
fi

# if we get here source browser.func
. /usr/share/dl_browser/browser.func
# puppyism
. /etc/DISTRO_SPECS

# package type
PKG_TYPE=${PKG_TYPE:-pet}
echo "Package type selected is $PKG_TYPE"
sleep 3

# Use the architecture of the current machine or whatever the user has
# set externally
ARCH=${ARCH:-$(uname -m)}
case "$ARCH" in
  x86_64) DEBARCH=amd64; LIBDIRSUFFIX="64" 
		[ -z "$DISTRO_ARCHDIR" ] && LIBDIRSUFFIX="";; # puppyism
    i?86) DEBARCH=i386;  LIBDIRSUFFIX="" ;;
       *) echo "The architecture $ARCH is not supported." >&2 ; exit 1 ;;
esac

# Work out the latest stable Opera if VERSION is unset
VERSION=$(wget -qO- http://deb.opera.com/opera/dists/stable/non-free/binary-$DEBARCH/Packages.gz | gzip -d | grep -A1 -x "Package: opera-stable" | sed -n "/Version/s/.* //p")

# Error out if $VERISON is unset, e.g. because previous command failed
if [ -z $VERSION ]; then
  echo "Could not work out the latest version; exiting" >&2
  exit 1
fi
echo "new: $VERSION"

CWD=$(pwd)
TMP=${TMP:-/tmp}
OPERA_DEB=opera-stable_${VERSION}_${DEBARCH}.deb

REPACKDIR=$TMP/repackage-opera-stable

# Since packaging is about to begin errors become more important now,
# so exit if things fail.
set -eu

# If the repackage is already present from the past, clear it down
# and re-create it.
if [ -d "$REPACKDIR" ]; then
  rm -fr "$REPACKDIR"
fi

mkdir -p "$REPACKDIR"/pkg/src
mkdir -p "$REPACKDIR"/src
sync

# Check if the current directory contains the Opera binary package,
# otherwise download it.
if [ -e "$OPERA_DEB" ]; then
  cp $OPERA_DEB "$REPACKDIR/src/"
else
  wget -P "$REPACKDIR/src" http://deb.opera.com/opera/pool/non-free/o/opera-stable/$OPERA_DEB
fi

# Now we have all the sources in place, switch to the package directory
# and start setting things up.
cd "$REPACKDIR/pkg"

# Extract the contents of the Opera binary package
ar p "../src/$OPERA_DEB" data.tar.xz | xz -d | tar -xf- \
  --transform="s,/lib/.*-linux-gnu,/lib$LIBDIRSUFFIX," \
  --exclude="./usr/share/lintian" --exclude="./usr/share/menu"

# fix.desktop
sed -i -e 's/Icon=.*/Icon=opera.svg/' \
		-e "s/Exec=/Exec=run-as-spot /" \
		-e 's/ %U//' \
		-e 's/Categories=.*/Categories=WebBrowser;/' \
		-e '/TryExec=.*/d' \
		usr/share/applications/*.desktop

# Make sure the file permissions are ok
chmod -R u+w,go+r-w,a-s .

# opera_sandbox needs to be setuid root for Opera to run
chmod 4711 usr/lib$LIBDIRSUFFIX/opera/opera_sandbox

# package as pet or sfs
( 
cd ../ 
if [ "$PKG_TYPE" = "pet" ]; then
  p=opera-${VERSION}-${ARCH}
  mv pkg ${p}
  chmod 755 $p # precaution
  DESC="Opera is a web browser based on the blink engine"
  DEPS=""
  CAT=Internet
  echo "Please wait while constructing ${p}.pet ..."
  dir2pet -x -s -w="$DESC" -d="$DEPS" -c="$CAT" -p=${p} 2>&1 >/dev/null
  cp ${p}.pet /tmp
else # sfs
  mksquashfs pkg opera-${VERSION}-${ARCH}.sfs
  cp opera-${VERSION}-${ARCH}.sfs /tmp
fi
)

# cleanup
cd ~
rm -r $REPACKDIR

# And ... we're done! ;)
echo 'Opera was successfully packaged'
exit
